/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.fleet.vehicle.wizard;

import java.awt.Component;
import java.util.HashSet;
import java.util.Set;
import javax.swing.ImageIcon;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import nl.cloudfarming.client.fleet.model.VehicleService;
import nl.cloudfarming.client.util.swing.ASBodyPanel;
import org.netbeans.api.project.Project;
import org.openide.WizardDescriptor;
import org.openide.WizardValidationException;
import org.openide.util.*;

@NbBundle.Messages({"invalidNameError=Invalid Name", "duplicateNameError=Name already exists"})
public class VehicleWizardPanel implements WizardDescriptor.Panel<WizardDescriptor> {

    /**
     * The visual component that displays this panel. If you need to access the
     * component from this class, just use getComponent().
     */
    private VehicleVisualPanel bodyPanel;
    private ASBodyPanel component;
    private ImageIcon icon = ImageUtilities.loadImageIcon("nl/cloudfarming/client/fleet/vehicle/vehicle32.png", true);
    private Project project;
    private final Set<ChangeListener> listeners = new HashSet<>(1);
    private WizardDescriptor wizardDescriptor;

    public VehicleWizardPanel(Project project) {
        this.project = project;
    }

    @Override
    public Component getComponent() {
        if (bodyPanel == null) {
            bodyPanel = new VehicleVisualPanel(this);
            component = new ASBodyPanel(bodyPanel);
            component.setName(Bundle.name());
            component.setIcon(icon);
        }
        return component;
    }

    @Override
    public HelpCtx getHelp() {
        return HelpCtx.DEFAULT_HELP;
    }

    @Override
    public boolean isValid() {
        String errorMsg = null;
        try {
            validate();
        } catch (WizardValidationException e) {
            errorMsg = e.getLocalizedMessage();
        } finally {
            wizardDescriptor.putProperty(WizardDescriptor.PROP_ERROR_MESSAGE, errorMsg);
            return errorMsg == null;
        }
    }

    @Override
    public void addChangeListener(ChangeListener l) {
        synchronized (listeners) {
            listeners.add(l);
        }
    }

    @Override
    public void removeChangeListener(ChangeListener l) {
        synchronized (listeners) {
            listeners.remove(l);
        }
    }

    protected final void fireChangeEvent() {
        Set<ChangeListener> ls;
        synchronized (listeners) {
            ls = new HashSet<>(listeners);
        }
        ChangeEvent ev = new ChangeEvent(this);
        for (ChangeListener l : ls) {
            l.stateChanged(ev);
        }
    }

    @Override
    public void readSettings(WizardDescriptor wiz) {
        this.wizardDescriptor = wiz;
        wiz.putProperty(WizardDescriptor.PROP_ERROR_MESSAGE, "wiz");
        bodyPanel.getNameField().setText(NbPreferences.forModule(VehicleWizardPanel.class).get("namePreference", ""));
    }

    @Override
    public void storeSettings(WizardDescriptor wiz) {
        wiz.putProperty("name", bodyPanel.getNameField().getText());
    }

    public void validate() throws WizardValidationException {
        String name = bodyPanel.getNameField().getText().trim();
        if (name.equals("")) {
            throw new WizardValidationException(null, Bundle.invalidNameError(), null);
        } else if (vehicleExists(name)) {
            throw new WizardValidationException(null, Bundle.duplicateNameError(), null);
        }
    }

    private boolean vehicleExists(String name) {
        VehicleService service = Lookup.getDefault().lookup(VehicleService.class);
        return service.getMachine(project, name) != null;
    }
}
